@@ -1,43 +1,10 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
-import copy
 from trytond.model import ModelView, ModelSQL, fields
-from trytond.model.modelstorage import OPERATORS
-from trytond.pyson import Not, Bool, Eval, Get, Equal
+from trytond.pyson import Eval
+from trytond.backend import TableHandler
 from trytond.transaction import Transaction
-
-
-class TimesheetWork(ModelSQL, ModelView):
-    _name = 'timesheet.work'
-
-    timesheet_lines = fields.One2Many('timesheet.line', 'work',
-            'Timesheet Lines',
-            depends=['timesheet_available', 'active'],
-            states={
-                'invisible': Not(Bool(Eval('timesheet_available'))),
-                'readonly': Not(Bool(Eval('active'))),
-            })
-    sequence = fields.Integer('Sequence')
-
-    def __init__(self):
-        super(TimesheetWork, self).__init__()
-        self._order.insert(0, ('sequence', 'ASC'))
-
-        self.parent = copy.copy(self.parent)
-        self.parent.context = copy.copy(self.parent.context)
-        self.parent.context['type'] = Eval('type')
-        self._reset_columns()
-
-    def copy(self, ids, default=None):
-        if default is None:
-            default = {}
-        default = default.copy()
-        if 'timesheet_lines' not in default:
-            default['timesheet_lines'] = False
-        return super(TimesheetWork, self).copy(ids, default=default)
-
-TimesheetWork()
+from trytond.pool import Pool
 
 
 class Work(ModelSQL, ModelView):
@@ -52,22 +19,19 @@ class Work(ModelSQL, ModelView):
             ('project', 'Project'),
             ('task', 'Task')
             ],
-            'Type', required=True, select=1,
-            states={
-                'invisible': Get(Eval('context', {}), 'type', False),
-            })
+        'Type', required=True, select=True)
     party = fields.Many2One('party.party', 'Party',
-            states={
-                'invisible': Not(Equal(Eval('type'), 'project')),
+        states={
+            'invisible': Eval('type') != 'project',
             }, depends=['type'])
     party_address = fields.Many2One('party.address', 'Contact Address',
-            domain=[('party', '=', Eval('party'))],
-            states={
-                'invisible': Not(Equal(Eval('type'), 'project')),
+        domain=[('party', '=', Eval('party'))],
+        states={
+            'invisible': Eval('type') != 'project',
             }, depends=['party', 'type'])
     effort = fields.Float("Effort",
-            states={
-                'invisible': Not(Equal(Eval('type'), 'task')),
+        states={
+            'invisible': Eval('type') != 'task',
             }, depends=['type'], help="Estimated Effort for this work")
     total_effort = fields.Function(fields.Float('Total Effort',
         help="Estimated total effort for this work and the sub-works"),
@@ -79,20 +43,42 @@ class Work(ModelSQL, ModelView):
     state = fields.Selection([
             ('opened', 'Opened'),
             ('done', 'Done'),
-            ], 'State',
-            states={
-                'invisible': Not(Equal(Eval('type'), 'task')),
-                'required': Equal(Eval('type'), 'task'),
-            }, select=1, depends=['type'])
+            ], 'State', required=True, select=True)
+    sequence = fields.Integer('Sequence', required=True)
 
     def default_type(self):
-        if Transaction().context.get('type') == 'project':
-            return 'project'
         return 'task'
 
     def default_state(self):
         return 'opened'
 
+    def default_effort(self):
+        return 0.0
+
+    def init(self, module_name):
+        timesheet_work_obj = Pool().get('timesheet.work')
+        cursor = Transaction().cursor
+        table_project_work = TableHandler(cursor, self, module_name)
+        table_timesheet_work = TableHandler(cursor, timesheet_work_obj,
+            module_name)
+        migrate_sequence = (not table_project_work.column_exist('sequence')
+            and table_timesheet_work.column_exist('sequence'))
+
+        super(Work, self).init(module_name)
+
+        # Migration from 2.0: copy sequence from timesheet to project
+        if migrate_sequence:
+            cursor.execute(
+                'SELECT t.sequence, t.id '
+                'FROM "%s" AS t '
+                'JOIN "%s" AS p ON (p.work = t.id)' % (
+                    timesheet_work_obj._table, self._table))
+            for sequence, id_ in cursor.fetchall():
+                sql = ('UPDATE "%s" '
+                        'SET sequence = %%s '
+                        'WHERE work = %%s' % self._table)
+                cursor.execute(sql, (sequence, id_))
+
     def __init__(self):
         super(Work, self).__init__()
         self._sql_constraints += [
@@ -100,16 +86,19 @@ class Work(ModelSQL, ModelView):
                  'timesheet work by task/project!'),
         ]
         self._order.insert(0, ('sequence', 'ASC'))
-        if 'company' in self._inherit_fields:
-            company_field = copy.copy(
-                self._inherit_fields['company'])
-            company_field[2].states = copy.copy(company_field[2].states)
-            company_field[2].states.update({
-                'invisible': Bool(Get(Eval('context', {}), 'type')),
-                'readonly': Not(Bool(Eval('active'))),
-            })
+        self._constraints += [
+            ('check_state',
+                'A work can not be closed if its children are still opened'),
+            ]
 
-            self._inherit_fields['company'] = company_field
+    def check_state(self, ids):
+        for work in self.browse(ids):
+            if ((work.state == 'opened'
+                        and (work.parent and work.parent.state == 'done'))
+                    or (work.state == 'done'
+                        and any(c.state == 'opened' for c in work.children))):
+                return False
+        return True
 
     def get_parent(self, ids, name):
         res = dict.fromkeys(ids, None)
@@ -137,7 +126,7 @@ class Work(ModelSQL, ModelView):
         return res
 
     def set_parent(self, ids, name, value):
-        timesheet_work_obj = self.pool.get('timesheet.work')
+        timesheet_work_obj = Pool().get('timesheet.work')
         if value:
             project_works = self.browse(ids + [value])
             child_timesheet_work_ids = [x.work.id for x in project_works[:-1]]
@@ -145,14 +134,14 @@ class Work(ModelSQL, ModelView):
         else:
             child_project_works = self.browse(ids)
             child_timesheet_work_ids = [x.work.id for x in child_project_works]
-            parent_timesheet_work_id = False
+            parent_timesheet_work_id = None
 
         timesheet_work_obj.write(child_timesheet_work_ids, {
                 'parent': parent_timesheet_work_id
                 })
 
     def search_parent(self, name, domain=None):
-        timesheet_work_obj = self.pool.get('timesheet.work')
+        timesheet_work_obj = Pool().get('timesheet.work')
 
         project_work_domain = []
         timesheet_work_domain = []
@@ -167,7 +156,8 @@ class Work(ModelSQL, ModelView):
         # we need to convert them to timesheet_work ids
         operands = set()
         for _, _, operand in timesheet_work_domain:
-            if isinstance(operand, (int, long)) and not isinstance(operand, bool):
+            if (isinstance(operand, (int, long))
+                    and not isinstance(operand, bool)):
                 operands.add(operand)
             elif isinstance(operand, list):
                 for o in operand:
@@ -218,7 +208,7 @@ class Work(ModelSQL, ModelView):
         id2work = {}
         leafs = set()
         for work in works:
-            res[work.id] = work.effort
+            res[work.id] = work.effort or 0
             id2work[work.id] = work
             if not work.children:
                 leafs.add(work.id)
@@ -242,7 +232,7 @@ class Work(ModelSQL, ModelView):
         return res
 
     def copy(self, ids, default=None):
-        timesheet_work_obj = self.pool.get('timesheet.work')
+        timesheet_work_obj = Pool().get('timesheet.work')
 
         int_id = isinstance(ids, (int, long))
         if int_id:
@@ -268,7 +258,7 @@ class Work(ModelSQL, ModelView):
         return new_ids
 
     def delete(self, ids):
-        timesheet_work_obj = self.pool.get('timesheet.work')
+        timesheet_work_obj = Pool().get('timesheet.work')
 
         if isinstance(ids, (int, long)):
             ids = [ids]
